# -*- coding: utf-8 -*-
from test import test_support as support
import unittest
import rlcompleter
import __builtin__ as builtins

"""
See comments for Completer.dict_key_matches() in rlcompleter.py regarding unicode

"""

class CompleteMe(object):
    """ Trivial class used in testing rlcompleter.Completer. """
    spam = 1


# A trivial dicts used in testing rlcompleter.Completer

NoUnicodeDictCompleteMe = { 'spam' : 1,
                           1984 : 42 } 

UnicodeDictCompleteMe = { 'spam' : 1,
                         1984 : 42,
                         u'öh, вау!' : 'yep world is not ascii only'} 


class TestRlcompleter(unittest.TestCase):
    def setUp(self):
        self.stdcompleter = rlcompleter.Completer()
        self.completer = rlcompleter.Completer(dict(spam=int,
                                        egg=str,
                                        CompleteMe=CompleteMe,
                                        UnicodeDictCompleteMe=UnicodeDictCompleteMe,
                                        NoUnicodeDictCompleteMe=NoUnicodeDictCompleteMe))

        # forces stdcompleter to bind builtins namespace
        self.stdcompleter.complete('', 0)

    def test_namespace(self):
        class A(dict):
            pass
        class B(list):
            pass

        self.assertTrue(self.stdcompleter.use_main_ns)
        self.assertFalse(self.completer.use_main_ns)
        self.assertFalse(rlcompleter.Completer(A()).use_main_ns)
        self.assertRaises(TypeError, rlcompleter.Completer, B((1,)))

    def test_global_matches(self):
        # test with builtins namespace
#        self.assertEqual(self.stdcompleter.global_matches('di'),
#                         [x+'(' for x in dir(builtins) if x.startswith('di')])
        self.assertEqual(self.stdcompleter.global_matches('st'),
                         [x + '(' for x in dir(builtins) if x.startswith('st')])
        self.assertEqual(self.stdcompleter.global_matches('akaksajadhak'), [])

        # test with a customized namespace
        self.assertEqual(self.completer.global_matches('CompleteM'),
                         ['CompleteMe('])
        self.assertEqual(self.completer.global_matches('eg'),
                         ['egg('])
        # XXX: see issue5256
        self.assertEqual(self.completer.global_matches('CompleteM'),
                         ['CompleteMe('])


    def test_attr_matches(self):
        # test with builtins namespace
        self.assertEqual(self.stdcompleter.attr_matches('str.s'),
                         ['str.%s(' % x for x in dir(str) if x.startswith('s')])

        self.assertEqual(self.stdcompleter.attr_matches('tuple.foospamegg'), [])

        # test with a customized namespace
        self.assertEqual(self.completer.attr_matches('CompleteMe.sp'),
                         ['CompleteMe.spam'])
        self.assertEqual(self.completer.attr_matches('Completeme.egg'), [])

        CompleteMe.me = CompleteMe
        self.assertEqual(self.completer.attr_matches('CompleteMe.me.me.sp'),
                         ['CompleteMe.me.me.spam'])
        self.assertEqual(self.completer.attr_matches('egg.s'),
                         ['egg.%s(' % x for x in dir(str)
                          if x.startswith('s')])

    def test_dict_key_matches_without_unicode(self):
        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe['),
                         ['NoUnicodeDictCompleteMe[1984]',
                          'NoUnicodeDictCompleteMe[\'spam\']'])
        
        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe[1'),
                         ['NoUnicodeDictCompleteMe[1984]'])
        
        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe[2'),
                         [])

        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe[s'),
                         [])

        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe[\''),
                         ['NoUnicodeDictCompleteMe[\'spam\']'])

        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe[\'s'),
                         ['NoUnicodeDictCompleteMe[\'spam\']'])

        self.assertEqual(self.completer.dict_key_matches('NoUnicodeDictCompleteMe[\"s'),
                         ['NoUnicodeDictCompleteMe[\"spam\"]'])


    def test_dict_key_matches_with_unicode(self):
        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe['),
                         ['UnicodeDictCompleteMe[1984]',
                          'UnicodeDictCompleteMe[\'spam\']',
                          u'UnicodeDictCompleteMe[u\'öh, вау!\']' ])
        
        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[1'),
                         ['UnicodeDictCompleteMe[1984]'])
        
        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[2'),
                         [])

        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[s'),
                         [])

        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[\''),
                         ['UnicodeDictCompleteMe[\'spam\']',
                          u'UnicodeDictCompleteMe[u\'öh, вау!\']'])

        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[\'s'),
                         ['UnicodeDictCompleteMe[\'spam\']'])

        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[\"s'),
                         ['UnicodeDictCompleteMe[\"spam\"]'])

        self.assertEqual(self.completer.dict_key_matches(u'UnicodeDictCompleteMe[u\'ö'),
                         [u'UnicodeDictCompleteMe[u\'öh, вау!\']'])

        self.assertEqual(self.completer.dict_key_matches('UnicodeDictCompleteMe[u\''),
                         ["UnicodeDictCompleteMe[u'spam']", #  it is a question of requirements!
                          u'UnicodeDictCompleteMe[u\'öh, вау!\']' ])

def test_main():
    support.run_unittest(TestRlcompleter)


if __name__ == '__main__':
    test_main()
